import { DatePipe } from '@angular/common';
import { HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { BrowserModule } from '@angular/platform-browser';
import { RouteReuseStrategy } from '@angular/router';
import { FileOpener } from '@awesome-cordova-plugins/file-opener/ngx';
import { Device } from '@ionic-native/device/ngx';
import { File } from '@ionic-native/file/ngx';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { LocalNotifications } from '@ionic-native/local-notifications/ngx';
import { Network } from '@ionic-native/network/ngx';
import { SplashScreen } from '@ionic-native/splash-screen/ngx';
import { StatusBar } from '@ionic-native/status-bar/ngx';
import { IonicModule, IonicRouteStrategy } from '@ionic/angular';
import { IonicStorageModule } from '@ionic/storage-angular';
import { RxReactiveFormsModule } from '@rxweb/reactive-form-validators';
import { FCM } from 'cordova-plugin-fcm-with-dependecy-updated/ionic/ngx';
import { NgxUiLoaderHttpModule, NgxUiLoaderModule, NgxUiLoaderRouterModule } from 'ngx-ui-loader';

import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { CoreModule } from './core/core.module';
import { MacsHttpInterceptor } from './interceptor';
import { TransformPipe } from './pipe';
import { ngxUiLoaderConfig, ngxUiLoaderHttpConfig } from './utility/configs';

@NgModule({
	declarations: [AppComponent, TransformPipe],
	entryComponents: [],
	imports: [
		BrowserModule,
		IonicModule.forRoot(),
		IonicStorageModule.forRoot(),
		HttpClientModule,
		AppRoutingModule,
		FormsModule,
		ReactiveFormsModule,
		RxReactiveFormsModule,
		NgxUiLoaderModule.forRoot(ngxUiLoaderConfig),
		NgxUiLoaderRouterModule,
		NgxUiLoaderHttpModule.forRoot(ngxUiLoaderHttpConfig),
		CoreModule,
	],
	providers: [
		StatusBar,
		SplashScreen,
		InAppBrowser,
		DatePipe,
		Network,
		FCM,
		/* process.env.NODE_ENV === "production"
	  ? FCM
	  : {
		  provide: FCM,
		  useValue: {
			getToken: () =>
			  Promise.resolve(
				"fwQIJqgYTzE:APA91bEbUjBRV5_g9fcyOZ-eh4WVt8OUZ8n42TzR26yXqnBeQ9eZk0wfz7GNaGbD9tY9U_oQ8h4kb2LxF57OivdU9kY3BRdmYPtRcEAWIbre3Li3Oje5V_WB0HqziJ384lmIUWVnoeX-"
			  ),
		  },
		}, // mock FCM like so, */
		Device,
		LocalNotifications,
		{ provide: RouteReuseStrategy, useClass: IonicRouteStrategy },
		{ provide: HTTP_INTERCEPTORS, useClass: MacsHttpInterceptor, multi: true },
		File,
		FileOpener
	],
	bootstrap: [AppComponent],
	exports: [TransformPipe],
})
export class AppModule { }
