import { DatePipe } from '@angular/common';
import { Component, Input, OnInit } from '@angular/core';
import { GpTemplatesCode, InfoUserProcedure, IterStatoPratica, ProcedureStatus, ProcedureTemplateConfig, VariableGp } from 'src/app/models';
import { GpValueVariablePipe } from 'src/app/pipe';
import { TEMPLATES_CODE, TEMPLATE_PROCEDURE } from 'src/app/utility';

@Component({
  selector: 'app-gp-procedure-iter',
  templateUrl: './gp-procedure-iter.component.html',
  styleUrls: ['./gp-procedure-iter.component.scss'],
})
export class GpProcedureIterComponent implements OnInit {
  @Input() variablesGp: VariableGp[];
  @Input() procedure: InfoUserProcedure;
  @Input() templateConfiguration: ProcedureTemplateConfig;
  @Input() procedureStatus: ProcedureStatus[];

  templatesCodes: GpTemplatesCode = TEMPLATES_CODE;
  templateProcedure = TEMPLATE_PROCEDURE;
  arrow_down: string = 'assets/icon/icona_sotto.svg';
  arrow_right: string = 'assets/icon/icona_avanti.svg';
  showCardContent: boolean;
  iconArrow: string;

  constructor(
    public gpValueVariablePipe: GpValueVariablePipe,
    private datePipe: DatePipe) {
    this.setUtility();
  }

  ngOnInit() {
  }

  setUtility() {
    this.iconArrow = this.arrow_right;
    setTimeout(() => {
      this.openCard();
    }, 300)
  }

  openCard() {
    this.showCardContent = true;
    this.iconArrow = this.arrow_down;
  }

  manageCloseOpen() {
    if (this.showCardContent) {
      this.showCardContent = false;
      this.iconArrow = this.arrow_right;
    } else {
      this.showCardContent = true;
      this.iconArrow = this.arrow_down;
    }
  }

  mappingDatiIter() {
    try {
      let itersStatoPratica: IterStatoPratica[] = JSON.parse(this.gpValueVariablePipe.transform(this.templateProcedure.codeVariables.statoPraticaJson, this.variablesGp) as string);
      itersStatoPratica.forEach((iter: IterStatoPratica) => {
        if (iter?.data)
          iter.data = this.datePipe.transform(iter.data, this.templateProcedure.pipeDateFormat.dataIter);
      });
      return itersStatoPratica && itersStatoPratica.length > 0 ? itersStatoPratica : [];
    } catch (e) {
      return [];
    }
  }

  }
