import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { IonSlides, Platform } from '@ionic/angular';
import { pluck } from 'rxjs/operators';
import { Tutorial } from 'src/app/models';
import { StorageService } from 'src/app/services';
import { IMAGE, MESSAGES, PAGES, TUTORIAL } from 'src/app/utility/global-constant';

@Component({
  selector: 'app-tutorial',
  templateUrl: './tutorial.page.html',
  styleUrls: ['./tutorial.page.scss'],
})
export class TutorialPage implements OnInit {
  titleHeader: string = 'Tutorial';
  showSkip = true;
  @ViewChild('slides', { static: true }) slides: IonSlides;
  lista_tutorial: Array<Tutorial>;
  textReady: string = MESSAGES.readyTutorial;
  logoHeader: string = IMAGE.logoHeader;
  widhtLogo: number = IMAGE.widthLogo;
  heightLogo: number = IMAGE.heightLogo;
  canBack: boolean; 
  ios: boolean;
  android: boolean;

  constructor(
    private platform: Platform,
    public router: Router,
    private activatedRoute: ActivatedRoute,
    public storageService: StorageService) {
      this.checkPlatform();
      this.getParams();
      this.setListaTutorial();
  }

  ngOnInit() {
  }

  ionViewWillEnter() {
    this.slides.update();
  }

  setListaTutorial() {
    this.lista_tutorial = TUTORIAL;
  }

  getParams() {
    this.activatedRoute.params.pipe(pluck('params'))
      .subscribe(
        (result) => {
          result? this.canBack = true: this.canBack = false;
        }
      );
  }

  checkPlatform() {
    this.ios = this.platform.is('ios');
    this.android = this.platform.is('android');
  }

  startApp() {
    this.router
      .navigateByUrl(PAGES.landing_page, { replaceUrl: true })
      .then(() => this.storageService.storeSawTutorial(true));
  }

  onSlideChangeStart(event) {
    event.target.isEnd().then(isEnd => {
      this.showSkip = !isEnd;
    });
  }

  checkCanBack(): boolean {
    return false;
  }

}
