import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { DomSanitizer, SafeResourceUrl } from '@angular/platform-browser';
import { IonSlides, ModalController } from '@ionic/angular';
import { VIDEOPLAYER } from 'src/app/utility/global-constant';
import { CONFIGSLIDE } from 'src/app/utility/slider.config';

import { IMAGE } from '../../../utility/global-constant';

@Component({
	selector: 'app-galleria-news',
	templateUrl: './galleria-news.page.html',
	styleUrls: ['./galleria-news.page.scss'],
})
export class GalleriaNewsPage implements OnInit {
	@Input() listaImage: Array<string>;
	@Input() listaVideoLocali: Array<string>;
	@Input() listaVideoEsterni: Array<string>;
	@Input() isImageGallery: boolean;
	@ViewChild(IonSlides) slides: IonSlides;
	slideOpts = CONFIGSLIDE;
	widthVideoPlayer: number = VIDEOPLAYER.width;
	heightVideoPlayer: number = VIDEOPLAYER.height;
	imageTemp: string = IMAGE.neLocandinaTemp;
	showAltImage: boolean = false;
	reloadImage: boolean = false;
	isOneElement: boolean = false;
	isLoadedImage: boolean = false;
	constructor(private modalController: ModalController, private sanitizer: DomSanitizer) {}

	ngOnInit() {
		if (this.isImageGallery) {
			this.isOneElement = this.listaImage?.length <= 1;
		} else {
			const totalLng = this.listaVideoLocali?.length + this.listaVideoEsterni?.length;
			if (totalLng) {
				this.isOneElement = totalLng <= 1;
			} else {
				this.isOneElement = true;
			}
		}
	}

	ionViewWillEnter() {
		this.slides.update();
		this.slides.lockSwipes(!this.isImageGallery);
	}

	close() {
		this.modalController.dismiss();
	}

	ionViewDidLeave() {
		/*   this.slides.update(); */
	}

	getSanitizerUrl(url: string): SafeResourceUrl {
		return this.sanitizer.bypassSecurityTrustResourceUrl('https://www.youtube.com/watch/v=bwFiRvVag8U');
	}

	slidePrev() {
		this.slides.lockSwipes(false);
		this.slides.slidePrev();
		if (!this.isImageGallery) {
			this.slides.lockSwipes(true);
		}
	}
	slideNext() {
		this.slides.lockSwipes(false);
		this.slides.slideNext();
		if (!this.isImageGallery) {
			this.slides.lockSwipes(true);
		}
	}

	loadAlternativeImage() {
		this.showAltImage = true;
		this.isLoadedImage = true;
	}

	loadedImage() {
		this.isLoadedImage = true;
	}
}
