import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Geolocation } from '@awesome-cordova-plugins/geolocation/ngx';
import { IonContent, Platform } from '@ionic/angular';
import { LayerGroup, Map, MapOptions, Marker } from 'leaflet';
import { NgxUiLoaderService } from 'ngx-ui-loader';
import { CategoriaPoi, PlacePortale, PlaceSearchRequest, PlaceTypePortale } from 'src/app/models';
import { GlobalService, PoiService, StorageService } from 'src/app/services';
import { UtilityService } from 'src/app/services/utility/utility.service';
import { htmlCodeToChar } from 'src/app/utility';
import { DURATIONTOAST, IMAGE, MESSAGES, PAGES, PAGINATION_POI, SEARCH } from 'src/app/utility/global-constant';

@Component({
	selector: 'app-poi',
	templateUrl: './poi.page.html',
	styleUrls: ['./poi.page.scss'],
})
export class PoiPage implements OnInit {
	titleHeader: string = PAGES.luoghi.title;
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
	@ViewChild(IonContent) content: IonContent;
	placeTypesPortale: PlaceTypePortale[] = [];
	showSkeletonPlaceTypesPortale: boolean = true;
	approxSizeTypesPlace: number = 3;
	showSkeletonPlacesPortale: boolean = true;
	approxSizePlaces: number = 3;
	placeTypeSelected: number;
	debounceSearch: number = SEARCH.debounce;
	titleSearched: string;
	page_number: number;
	neFinished: boolean;
	notResult: boolean = false;
	placesPortale: PlacePortale[];
	placesPortalePlus: PlacePortale[];
	ios: boolean;
	android: boolean;
	textMessage: string = MESSAGES.notPlaces;

	constructor(
		private platform: Platform,
		private poiService: PoiService,
		private router: Router,
		private activatedRoute: ActivatedRoute,
		private globalService: GlobalService,
		private storageService: StorageService,
	) {
		this.checkPlatform();
		this.setUtility();
		this.getPlaceTypePortale();
	}

	setUtility() {
		this.placesPortale = [];
		this.placesPortalePlus = [];
		this.setInitPagination();
		this.titleSearched = '';
	}

	ngOnInit() {
	}

	checkPlatform() {
		this.ios = this.platform.is('ios');
		this.android = this.platform.is('android');
	}

	setDefaultTypePlaces() {
		this.placeTypeSelected = this.placeTypesPortale[0].value;
		// this.placeTypeSelected = this.placeTypesPortale[3].value;
	}

	searchPlacesByTipologia(event: Event) {
		this.placeTypeSelected = event['detail']['value'] as number;
		this.titleSearched = '';
		this.setInitPagination();
		this.forceContent();
		this.getPlacesFiltered();
		return true;
	}

	searchPlaceByTitle(event: Event) {
		this.setInitPagination();
		this.titleSearched = event['detail']['value'] as string;
		this.forceContent();
		this.getPlacesFiltered();
		return true;
	}

	setInitPagination() {
		this.page_number = PAGINATION_POI.start_page;
		this.neFinished = false;
	}

	setEndPagination() {
		this.neFinished = true;
	}

	forceContent() {
		this.content.scrollToTop();
	}

	doRefresh(event) {
		this.setInitPagination();
		this.getPlacesFiltered();
		setTimeout(() => {
			event.target.complete();
		}, 250);
	}

	doInfinite(event) {
		if (!this.neFinished && !this.showSkeletonPlacesPortale && !this.showSkeletonPlaceTypesPortale) {
			this.getPlacesFiltered(true);
			event.target.complete();
		} else {
			event.target.complete();
		}

	}

	detailPlace(place: PlacePortale) {
		this.storePlacePortal(place);
		this.router.navigate([PAGES.dettaglioLuogo.link], { relativeTo: this.activatedRoute });
	}

	setCardMessage(zeroResult: boolean) {
		if (zeroResult) {
			this.notResult = true;
			this.placesPortale = [];
		}
	}

	normalizeRows(placesPortale: PlacePortale[]) {
		placesPortale.forEach((place) => {
			place.title = htmlCodeToChar(place.title);
		});
	}

	upgradePlacesPortale(placesPortalePlus: PlacePortale[]) {
		for (let place of placesPortalePlus) {
			this.placesPortale.push(place);
		}
		this.placesPortalePlus = [];
	}

	storePlacePortal(place: PlacePortale) {
		this.storageService.storePlaceToShow(place);
	}

	getPlacesFiltered(plusElements?: boolean) {
		this.getPlacesFilteredService(this.prepareRequestLuoghiFiltered(plusElements), plusElements);
	}

	prepareRequestLuoghiFiltered(plusElements?: boolean): PlaceSearchRequest {
		this.page_number = plusElements ? this.page_number + 1 : this.page_number;
		return {
			titolo: this.titleSearched,
			tipo_luogo: this.placeTypeSelected,
			perPage: PAGINATION_POI.items_per_page,
			page: this.page_number
		}
	}

	getPlaceTypePortale() {
		this.poiService.getPlacesType().subscribe(
			{
				next: (res) => {
					this.showSkeletonPlaceTypesPortale = false;
					if (res && res.data && res.data.length > 0) {
						this.placeTypesPortale = res.data;
						this.setDefaultTypePlaces();
						this.getPlacesFiltered();
					}
				},
				error: (err) => {
					this.showSkeletonPlaceTypesPortale = false;
					this.showSkeletonPlacesPortale = false;
					this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
				},
				complete: () => {
				}
			}
		);
	}

	getPlacesFilteredService(placeSearchRequest: PlaceSearchRequest, plusElements?: boolean) {
		this.poiService.getPlacesFiltered(placeSearchRequest).subscribe(
			{
				next: (result) => {
					this.showSkeletonPlacesPortale = false;
					if (result.data && result.data.length > 0) {
						this.normalizeRows(result.data);
						plusElements ? (this.placesPortalePlus = result.data) : (this.placesPortale = result.data);
					} else {
						plusElements ? this.setEndPagination() : this.setCardMessage(true);
					}
				},
				error: (error) => {
					this.showSkeletonPlacesPortale = false;
					this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
					this.setEndPagination();
					this.setCardMessage(true);
				},
				complete: () => {
					this.upgradePlacesPortale(this.placesPortalePlus);
				},
			}
		);
	}
}