import { DatePipe, Location } from '@angular/common';
import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';
import { BehaviorSubject } from 'rxjs';
import { PrenotazioneAnagraficaFormComponent } from 'src/app/forms/prenotazione-anagrafica-form/prenotazione-anagrafica-form.component';
import { PrenotazioneAnagraficaFormModel } from 'src/app/forms/prenotazione-anagrafica-form/prenotazione-anagrafica-form.model';
import { Anagrafica, PSte1Step2, UserInfo } from 'src/app/models';
import { GlobalService, StorageService } from 'src/app/services';
import { DURATIONTOAST, IMAGE, MESSAGES, PAGES, RESERVATION } from 'src/app/utility/global-constant';

@Component({
	selector: 'app-prenotazione-step1',
	templateUrl: './prenotazione-step1.page.html',
	styleUrls: ['./prenotazione-step1.page.scss'],
})
export class PrenotazioneStep1Page implements OnInit {
	@ViewChild(PrenotazioneAnagraficaFormComponent)
	paForm: PrenotazioneAnagraficaFormComponent;
	userInfo$: BehaviorSubject<UserInfo>;
	userInfo: UserInfo;
	titleForWho: string = RESERVATION.titleForWho.toLocaleUpperCase();
	forMeValue: string = RESERVATION.forMeValue;
	forMeLabel: string = RESERVATION.forMeLabel;
	forOthersValue: string = RESERVATION.forOthersValue;
	forOthersLabel: string = RESERVATION.forOthersLabel;
	checkChoose: string = RESERVATION.forMeValue;
	titleHeader: string = PAGES.prenotazioneStep1.title;
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;

	/*   canBack: boolean; */
	constructor(
		private globalService: GlobalService,
		private storageService: StorageService,
		private router: Router,
		public location: Location,
		private activatedRoute: ActivatedRoute,
		private datePipe: DatePipe
	) {
		this.initializeSubject();
		this.subscribeSubject();
		this.getUserLogged();
	}

	ngOnInit() {}

	ngAfterViewInit(): void {
		this.setUtility();
		this.paForm.creaForm(new PrenotazioneAnagraficaFormModel());
	}

	ionViewWillEnter() {
		this.storageService.removeInfoStep2();
    }

	setUtility() {}

	setForm() {
		this.paForm.codice_fiscale = this.userInfo.sub.toLocaleUpperCase();
		this.paForm.nome = this.userInfo.firstName;
		this.paForm.cognome = this.userInfo.lastName;
		this.paForm.email = this.userInfo.email;
		this.paForm.sesso = this.userInfo.gender;
		this.paForm.data_nascita = this.userInfo.birthdate;
		this.paForm.disablePartialForm();
	}

	goToNext() {
		if (this.globalService.validateForm(this.paForm.form)) {
			if ((this.paForm.prefix && !this.paForm.telefono) || (!this.paForm.prefix && this.paForm.telefono)) {
				this.globalService.presentToast(MESSAGES.missingTelOrPrefix, DURATIONTOAST.default);
			} else {
				if (!!this.paForm.telefono || !!this.paForm.cellulare || !!this.paForm.email) {
					let params = this.globalService.encodeParams({
						anagraficaPresentante: this.checkChoose === this.forMeValue ? this.prepareAnagrafica(1) : this.prepareAnagrafica(0),
						anagraficaFruitore: this.prepareAnagrafica(1),
					} as PSte1Step2);

					this.router.navigate([PAGES.prenotazioneStep2.link, params], {
						relativeTo: this.activatedRoute,
					});
				} else {
					this.globalService.presentToast(MESSAGES.missingTelOrCelOrEmail, DURATIONTOAST.default);
				}
			}
		} else {
			this.globalService.presentToast(MESSAGES.missingFields, DURATIONTOAST.default);
		}
	}

	getUserLogged() {
		this.storageService.getUserInfo().then((userInfo: UserInfo) => {
			this.userInfo$.next(userInfo);
		});
	}

	initializeSubject() {
		this.userInfo$ = new BehaviorSubject<UserInfo>(null);
	}

	subscribeSubject() {
		this.userInfo$.subscribe((userInfo) => {
			this.userInfo = userInfo ? userInfo : null;
			if (userInfo) {
				this.userInfo = userInfo;
				this.setForm();
			}
		});
	}

	handleForWho(event: any) {
		this.checkChoose = event.detail.value;
		switch (this.checkChoose) {
			case this.forMeValue:
				this.setForm();
				this.paForm.disablePartialForm();
				break;
			case this.forOthersValue:
				this.paForm.resetForm();
				this.paForm.enableForm();
				break;
			default:
				break;
		}
		/*     if (scelto === 'per_me') {
          this.visualizzaForm = true;
          this.setUserForm();
          this.disableForm();
          this.abilitaButton = true;
          this.passOk = true;
        } else {
          this.isPerTerzi = true;
          this.visualizzaForm = true;
          this.enableForm();
          this.resetForm();
          this.abilitaButton = true;
        } */
	}

	prepareAnagrafica(typeAnagrafica: number) {
		//0 -> Presentante
		//1 -> Fruitore
		let anagrafica: Anagrafica;
		switch (typeAnagrafica) {
			case 0:
				anagrafica = {
					nome: this.userInfo.firstName,
					cognome: this.userInfo.lastName,
					sesso: this.userInfo.gender,
					dataNascita: moment(this.userInfo.birthdate).format('DD/MM/yyyy'),
					codiceCatasto: this.estrapolaCodiceCatasto(this.userInfo.sub.toLocaleUpperCase()),
					cf: this.userInfo.sub.toLocaleUpperCase(),
					contatti: {
						email: this.userInfo.email,
						telefonoFisso: undefined,
						telefonoCellulare: undefined,
					},
				};
				break;
			case 1:
				anagrafica = {
					nome: this.paForm.nome,
					cognome: this.paForm.cognome,
					sesso: this.paForm.sesso,
					dataNascita: this.transformDateToString(new Date(this.paForm.data_nascita)),
					codiceCatasto: this.estrapolaCodiceCatasto(this.paForm.codice_fiscale?.toUpperCase()),
					cf: this.paForm.codice_fiscale?.toUpperCase(),
					contatti: {
						email: this.paForm.email,
						telefonoFisso: this.paForm.prefix ? `${this.paForm.prefix}-${this.paForm.telefono}` : undefined,
						telefonoCellulare: this.paForm.cellulare,
					},
				};
				break;
		}
		return anagrafica;
	}

	estrapolaCodiceCatasto(cf: string) {
		return cf.substring(11, 15);
	}

	transformDateToString(date: Date): string {
		return this.datePipe.transform(date, 'dd/MM/yyyy');
	}

	transformFormatString(stringDate: string) {
		let day = stringDate.substring(0, 2);
		let month = stringDate.substring(3, 2);
		let year = stringDate.substring(6, 4);
		return year + '-' + month + '-' + day;
	}
}
