import { Injectable } from '@angular/core';
import { FormGroup, ValidationErrors } from '@angular/forms';
import { Router } from '@angular/router';
import { FileOpener } from '@awesome-cordova-plugins/file-opener/ngx';
import { File } from '@ionic-native/file/ngx';
import { AlertButton, AlertController, Platform, ToastController } from '@ionic/angular';
import * as CryptoJS from 'crypto-js';
import { NgxUiLoaderService } from 'ngx-ui-loader';
import { CategoriaPoi, Domanda, Funzionalita, VariableGp } from 'src/app/models';

import { getMediaTypeFromPath } from '../../utility/global-constant';

const CACHE = 'CACHE';
const ENCRYPT_PARAMETERS = true;
const URI_ENCODE_PARAMETERS = false;

@Injectable({
	providedIn: 'root',
})
export class GlobalService {
	all_: string;
	mm: number;
	paramsSecret: string;
	cacheDate: Date;
	public readonly debug = true;

	constructor(
		private router: Router,
		public alertController: AlertController,
		public toastController: ToastController,
		private ngxService: NgxUiLoaderService,
		private file: File,
		private fileOpener: FileOpener,
		private platform: Platform) {
		this.all_ = '. '.repeat(1000);
		this.mm = 72 / 25.6;
		this.paramsSecret = '6QtxgFdnb9bBGm@7[s<A8h]X+%$B%FY&';
		this.cacheDate = new Date(0);
	}

	public decodeParams(sparams: string): any {
		if (sparams) {
			let params;
			if (ENCRYPT_PARAMETERS) {
				params = JSON.parse(CryptoJS.AES.decrypt(sparams, this.paramsSecret).toString(CryptoJS.enc.Utf8));
			} else if (URI_ENCODE_PARAMETERS) {
				params = JSON.parse(decodeURI(sparams));
			} else {
				params = JSON.parse(atob(sparams));
			}
			return params;
		}
		return {};
	}

	public encodeParams(params: any): string {
		this.removeEmpty(params);
		params[' '] = new Date().getTime();
		if (ENCRYPT_PARAMETERS) {
			return CryptoJS.AES.encrypt(JSON.stringify(params), this.paramsSecret).toString();
		}
		if (URI_ENCODE_PARAMETERS) {
			return encodeURI(JSON.stringify(params));
		}
		return btoa(JSON.stringify(params));
	}

	removeEmpty = (obj) => {
		Object.keys(obj).forEach((key) => {
			if (obj[key] && typeof obj[key] === 'object') {
				this.removeEmpty(obj[key]);
			} else if (obj[key] == null) {
				delete obj[key];
			} else if (obj[key] instanceof Array && obj[key].length === 0) {
				delete obj[key];
			}
		});
	};

	validateForm(form: FormGroup): boolean {
		if (!form.valid) {
			Object.keys(form.controls).forEach((field) => {
				const control = form.get(field);
				control.markAsTouched({ onlySelf: true });
			});
			if (this.debug) {
				this.logFormErrors(form);
			}
		}
		return form.valid;
	}

	validateForms(form: FormGroup[]): boolean {
		let valid = true;
		if (form.length < 1) {
			return;
		}
		for (let i = 0; i < form.length; i++) {
			if (form[i]) {
				Object.keys(form[i].controls).forEach((field) => {
					if (!form[i].valid) {
						valid = false;
					}
					const control = form[i].get(field);
					control.markAsTouched({ onlySelf: true });
				});
			}
		}
		return valid;
	}

	logFormErrors(form: FormGroup) {
		if (!form.valid) {
			Object.keys(form.controls).forEach((key) => {
				const controlErrors: ValidationErrors = form.get(key).errors;
				if (controlErrors) {
					Object.keys(controlErrors).forEach((keyError) => {
						//console.log('Key control: ' + key + ', keyError: ' + keyError + ', err value: ', controlErrors[keyError]);
					});
				}
			});
		}
	}

	getLocaleDateString(data: Date) {
		const formattedData = data.toISOString().split('T')[0];
		const formattedHour = data.toLocaleTimeString();
		return formattedData + 'T' + formattedHour + '.000Z';
	}

	//togliendo il time zone
	getRealDateSTM(data: Date): Date {
		let nuovaData = new Date(data);
		let hours: number = nuovaData.getHours();
		let timeZone: number = nuovaData.getTimezoneOffset();
		nuovaData.setHours(hours - timeZone / 60);
		return nuovaData;
	}

	//includendo il time zone
	getRealDateCTM(data: Date): Date {
		data.setHours(data.getHours() + data.getTimezoneOffset() / 60);
		return data;
	}

	convertBase64ToBlob(base64: string): Blob {
		let pos = base64.indexOf(';base64,');
		let type = base64.substring(5, pos);
		let b64 = base64.substr(pos + 8);
		// decode base64
		let imageContent = atob(b64);
		// create an ArrayBuffer and a view (as unsigned 8-bit)
		let buffer = new ArrayBuffer(imageContent.length);
		let view = new Uint8Array(buffer);
		// fill the view, using the decoded base64
		for (let n = 0; n < imageContent.length; n++) {
			view[n] = imageContent.charCodeAt(n);
		}
		// convert ArrayBuffer to Blob
		let blob = new Blob([buffer], { type: type });
		return blob;
	}

	b64toBlob(b64Data, contentType) {
		contentType = contentType || '';
		let sliceSize = 512;
		let byteCharacters = atob(b64Data);
		let byteArrays = [];

		for (let offset = 0; offset < byteCharacters.length; offset += sliceSize) {
			let slice = byteCharacters.slice(offset, offset + sliceSize);

			let byteNumbers = new Array(slice.length);
			for (let i = 0; i < slice.length; i++) {
				byteNumbers[i] = slice.charCodeAt(i);
			}
			let byteArray = new Uint8Array(byteNumbers);

			byteArrays.push(byteArray);
		}
		let blob = new Blob(byteArrays, { type: contentType });
		return blob;
	}

	async presentToast(message: string, duration: number) {
		const toast = await this.toastController.create({
			message: message,
			duration: duration,
			color: 'toast',
			cssClass: 'customToast',
		});
		toast.present();
	}

	checkCF(cf: string) {
		if (cf) {
			let patternCF = /^[a-zA-Z]{6}[0-9]{2}[a-zA-Z][0-9]{2}[a-zA-Z][0-9]{3}[a-zA-Z]$/;
			if (cf.search(patternCF) == -1) {
				return false;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}

	partition(array: Array<Domanda>, left: number = 0, right: number = array.length - 1) {
		const pivot = array[Math.floor((right + left) / 2)].ordinamento;
		let i = left;
		let j = right;

		while (i <= j) {
			while (array[i].ordinamento < pivot) {
				i++;
			}

			while (array[j].ordinamento > pivot) {
				j--;
			}

			if (i <= j) {
				[array[i], array[j]] = [array[j], array[i]];
				i++;
				j--;
			}
		}

		return i;
	}

	quickSort(array: Array<Domanda>, left: number = 0, right: number = array.length - 1) {
		let index;

		if (array.length > 1) {
			index = this.partition(array, left, right);

			if (left < index - 1) {
				this.quickSort(array, left, index - 1);
			}

			if (index < right) {
				this.quickSort(array, index, right);
			}
		}

		return array;
	}

	/*   compositeForRowCatPoi(listaCategoriePoiIN: Array<CategoriaPoi>, nForRow: number): Array<CategoriaPoi>[] {
	  const numbRows = listaCategoriePoiIN.length / nForRow;
	  let size = listaCategoriePoiIN.length;
	  let listaCategoriePoi: Array<CategoriaPoi> = [];
	  let listaCategoriePoiFR: Array<CategoriaPoi>[] = [];
	  let count = 0;
	  for (let i = 0; i < numbRows; i++) {
		listaCategoriePoi = [];
		for (let j = 0; j < nForRow && count < size; j++) {
		  listaCategoriePoi.push(listaCategoriePoiIN[count]);
		  count++;
		}
		listaCategoriePoiFR.push(listaCategoriePoi);
	  }
	  return listaCategoriePoiFR;
	} */

	compositeForRowCatPoi(listaCategoriePoiIN: Array<CategoriaPoi>, nForRow: number, maxRowFS: number): Array<CategoriaPoi>[][] {
		const numbRows = listaCategoriePoiIN.length / nForRow;
		let size = listaCategoriePoiIN.length;

		let listaCategoriePoi: Array<CategoriaPoi> = [];
		let listaCategoriePoiFR: Array<CategoriaPoi>[] = [];
		let listaCategoriePoiFRTemp: Array<CategoriaPoi>[] = [];
		let listaCategoriePoiFRR: Array<CategoriaPoi>[][] = [];
		let count = 0;
		for (let i = 0; i < numbRows; i++) {
			listaCategoriePoi = [];
			for (let j = 0; j < nForRow && count < size; j++) {
				listaCategoriePoi.push(listaCategoriePoiIN[count]);
				count++;
			}
			listaCategoriePoiFR.push(listaCategoriePoi);
		}

		count = 0;
		let sizeNewArray = listaCategoriePoiFR.length;
		let timesCycle: number = sizeNewArray / maxRowFS;
		for (let i = 0; i < timesCycle; i++) {
			listaCategoriePoiFRTemp = [];
			for (let j = 0; j < maxRowFS && count < sizeNewArray; j++) {
				listaCategoriePoiFRTemp.push(listaCategoriePoiFR[count]);
				count++;
			}
			listaCategoriePoiFRR.push(listaCategoriePoiFRTemp);
		}
		return listaCategoriePoiFRR;
	}

	compositeForBlockRowHome(listaFunzionalitaIN: Array<Funzionalita>, nForRow: number, maxRowFS: number): Array<Funzionalita>[][] {
		//funzionalità per block->righe
		let listaFunzionalitaFBR: Array<Funzionalita>[][] = [];
		let listaFunzionalita: Array<Funzionalita> = [];
		let listaFunzionalitaFR: Array<Funzionalita>[] = [];
		let listaFunzionalitaFRtemp: Array<Funzionalita>[] = [];

		let size = listaFunzionalitaIN.length;
		const numbRows = listaFunzionalitaIN.length / nForRow;
		let count = 0;
		for (let i = 0; i < numbRows; i++) {
			listaFunzionalita = [];
			for (let j = 0; j < nForRow && count < size; j++) {
				listaFunzionalita.push(listaFunzionalitaIN[count]);
				count++;
			}
			listaFunzionalitaFR.push(listaFunzionalita);
		}
		count = 0;
		let sizeNewArray = listaFunzionalitaFR.length;
		let timesCycle: number = sizeNewArray / maxRowFS;
		for (let i = 0; i < timesCycle; i++) {
			listaFunzionalitaFRtemp = [];
			for (let j = 0; j < maxRowFS && count < sizeNewArray; j++) {
				listaFunzionalitaFRtemp.push(listaFunzionalitaFR[count]);
				count++;
			}
			listaFunzionalitaFBR.push(listaFunzionalitaFRtemp);
		}
		return listaFunzionalitaFBR;
	}

	startNgxLoader() {
		this.ngxService.start();
	}

	stopNgxLoader() {
		this.ngxService.stopAll();
	}

	downloadFile(fileName: string, fileBase64: string | Blob | ArrayBuffer): Promise<any> {
		if (this.platform.is('ios')) {
			return this.file.writeFile(this.file.applicationStorageDirectory + '/Documents', `${new Date().getTime()}_${fileName}`, fileBase64, { replace: true });
		} else if (this.platform.is('android')) {
			return this.file.writeFile(this.file.externalRootDirectory + '/Download', `${new Date().getTime()}_${fileName}`, fileBase64, { replace: true });
		} else {
			return null;
		}
	}

	async presentAlertConfirm(header: string, message: string, buttons: AlertButton[]) {
		const alert = await this.alertController.create({
			cssClass: 'my-custom-class',
			header,
			message,
			buttons,
		});
		await alert.present();
	}

	openFile(filePath: string) {
		const mediaType: string = getMediaTypeFromPath(filePath);
		return this.fileOpener.open(filePath, mediaType);
	}

}
