package it.itsvil.macs.backend.service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import com.fasterxml.jackson.databind.ObjectMapper;
import it.itsvil.macs.backend.dto.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import it.itsvil.macs.backend.dao.ConfigurationMacsDao;
import it.itsvil.macs.backend.util.MacsEnum;
import lombok.extern.log4j.Log4j2;




@Log4j2
@Service(value = "ConfigurationMacsServiceImpl")
public class ConfigurationMacsServiceImpl implements ConfigurationMacsService {

	private final static ObjectMapper mapper = new ObjectMapper();

	@Autowired
	private ConfigurationMacsDao dao;

	@Autowired
	private PlatformTransactionManager transactionManager;

	@Value(value = "${macs.poi.configuration.path}")
	private String pathPoiConfigurations;

	@Override
	public boolean insertTipoNotifica(TipoNotificaDTO tipoNotifica) {
		log.info("START");
		boolean risultato=dao.insertTipoNotifica(tipoNotifica);
		log.info("END");
		return risultato;
	}

	@Override
	public List<TipoNotificaDTO> getAllTipoNotifiche() {
		log.info("START");
		List<TipoNotificaDTO> result = dao.getAllTipoNotifiche();
		log.info("END");
		return result;
	}

	@Override
	public TipoNotificaDTO getTipoNotificheById(String t_cod_notifiche) {
		log.info("START");
		TipoNotificaDTO result = dao.getTipoNotificheById(t_cod_notifiche);
		log.info("END");
		return result;
	}

	@Override
	public boolean noPushNotification(UserNotificheNegateDTO userNotificheNegateDTO) {
		log.info("START");
		boolean risultato=dao.noPushNotification(userNotificheNegateDTO);
		log.info("END");
		return risultato;
	}

	@Override
	public boolean yesPushNotification(UserNotificheNegateDTO userNotificheNegateDTO) {
		log.info("START");
		boolean risultato=dao.yesPushNotification(userNotificheNegateDTO);
		log.info("END");
		return risultato;
	}

	@Override
	public List<TipoNotificaDTO> noPushListForUser(String t_spid_code) {
		log.info("START");
		List<TipoNotificaDTO> result = dao.noPushListForUser(t_spid_code);
		log.info("END");
		return result;
	}

	@Override
	public List<NotificationForUserDTO> getListOfAllNotificationByUser(String t_spid_code) {
		log.info("START");

		List<TipoNotificaDTO> resultOfAllNotification = dao.getAllTipoNotifiche();
		if (null == resultOfAllNotification) {
			log.error("ERRORE: nessun notifica push presente in tabella tipologia push");
			return null;
		}

		List<TipoNotificaDTO> resultOfDenyPushByUser = dao.noPushListForUser(t_spid_code);
		if (null == resultOfDenyPushByUser) {
			log.error("ERRORE: nessun notifica push presente in tabella di push negate ");
			return null;
		}

		List<NotificationForUserDTO> result = new ArrayList<NotificationForUserDTO>();
		for (TipoNotificaDTO t1:resultOfAllNotification) {
			boolean flag = false;
			for (TipoNotificaDTO t2:resultOfDenyPushByUser) {
				if (t1.getT_cod_notifiche().equalsIgnoreCase(t2.getT_cod_notifiche())) {
					flag = true;
				}
			}
			NotificationForUserDTO push=new NotificationForUserDTO();
			push.setCodice(t1.getT_cod_notifiche());
			if (flag)
				push.setStato(MacsEnum.NO.getCodice());
			else push.setStato(MacsEnum.SI.getCodice());
			push.setTitolo(t1.getT_nome_notifiche());
			result.add(push);
		}

		log.info("END");
		return result;
	}

	@Override
	public boolean updateNotifichePerUtente(ListNotificationForUserRequestDTO request) {
		log.info("START");

		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {

			List<NotificationForUserDTO> list = request.getListaNotifiche();

			for (NotificationForUserDTO n:list) {
				UserNotificheNegateDTO userNotificheNegateDTO = new UserNotificheNegateDTO();
				userNotificheNegateDTO.setT_cod_notifiche(n.getCodice());
				userNotificheNegateDTO.setT_spid_code(request.getSpid_code());
				if (n.getStato().equalsIgnoreCase(MacsEnum.NO.getCodice())) { 
					if (!noPushNotification(userNotificheNegateDTO)) {
						log.error("ERRORE: Si è verificato un errore durante l'aggiornamento tutto sarà rollbeackato");
						transactionManager.rollback(txStatus);
						return false;
					}
				}
				else if (n.getStato().equalsIgnoreCase(MacsEnum.SI.getCodice())) {
					if (!yesPushNotification(userNotificheNegateDTO)) {
						log.error("ERRORE: Si è verificato un errore durante l'aggiornamento tutto sarà rollbeackato");
						transactionManager.rollback(txStatus);
						return false;
					}
				}
			}
		}catch (Exception e) {
			log.error("ERRORE: Si è verificato un errore durante l'aggiornamento tutto sarà rollbeackato");
			transactionManager.rollback(txStatus);
			return false;
		}

		log.info("END");
		transactionManager.commit(txStatus);

		return true;
	}

	@Override
	public List<PoiConfigDTO> getPoiCategories() throws IOException {
		log.info("path -> pathPoiConfigurations");
		URL url = new URL(pathPoiConfigurations);
		BufferedReader read = new BufferedReader(
				new InputStreamReader(url.openStream()));

		List<PoiConfigDTO> poiConfig = mapper.readValue(read, List.class);
		return poiConfig;
	}

}
