package it.itsvil.macs.backend.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import it.itsvil.macs.backend.client.NotificatoreClient;
import it.itsvil.macs.backend.client.ProfilatoreClient;
import it.itsvil.macs.backend.dao.DispositivoDao;
import it.itsvil.macs.backend.dao.UserDao;
import it.itsvil.macs.backend.dto.DispositivoDTO;
import it.itsvil.macs.backend.dto.GestorePraticaSendPushRequestDTO;
import it.itsvil.macs.backend.dto.LogoutDispositivoRequestDTO;
import it.itsvil.macs.backend.dto.NotificationInfoDTO;
import it.itsvil.macs.backend.dto.NotificatorePushBrodCastRequestDTO;
import it.itsvil.macs.backend.dto.NotificatorePushRequestDTO;
import it.itsvil.macs.backend.dto.NotificatorePushResponseDTO;
import it.itsvil.macs.backend.dto.NotificatoreTokenDTO;
import it.itsvil.macs.backend.dto.ProfilatoreInsertUtenzaByRuoloRequestDTO;
import it.itsvil.macs.backend.dto.ProfilatoreInsertUtenzaByRuoloResponseDTO;
import it.itsvil.macs.backend.dto.PunctualPushNotificationDTO;
import it.itsvil.macs.backend.dto.TokenFirebaseRequestDTO;
import it.itsvil.macs.backend.dto.UserDTO;
import it.itsvil.macs.backend.util.MacsEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "DispositivoServiceImpl")
public class DispositivoServiceImpl implements DispositivoService {

	@Autowired
	private DispositivoDao dao;

	@Autowired 
	private UserDao userDao;

	@Autowired
	private Environment env;

	@Autowired
	private NotificatoreClient client;
	
	@Autowired
	private ProfilatoreClient profilatoreClient;
	
	@Value(value = "${macs.profilatore.ruolo.cittadino}")
	private String ruoloCittadino;

	@Override
	public boolean sendToken(TokenFirebaseRequestDTO request) {
		log.info("START");
		
		boolean result=false;
		UserDTO resultUser = userDao.checkUser(request.getSpidCode());
		
		if (null==resultUser) {
			log.info("utente con spidcode: {} non e' presente nella tabella User", request.getSpidCode());
			
			UserDTO newUser = new UserDTO();
			newUser.setT_spid_code(request.getSpidCode());
			newUser.setT_codice_fiscale(request.getCodicefiscale());
			newUser.setD_data_ins(LocalDateTime.now());
			
			if (userDao.insertUser(newUser))
				if (insertOrUpdateDispositivo(request))
					result = true;
				else result = false;
			else result = false;
			
			//Inserimento utente in Profilatore
			if(Boolean.FALSE.equals(insertUtenzaByRuolo(request.getSpidCode()))) {
				log.info("Errore durante l'inserimento dell'utenza in profilatore");
				result = false;
			}
		}else {
			log.info("utente con spidcode: {} e' presente nella tabella User", request.getSpidCode());
			if (insertOrUpdateDispositivo(request))
				result = true;
			else result = false;
		}
		
		log.info("END");
		return result;
	}

	private boolean insertOrUpdateDispositivo(TokenFirebaseRequestDTO request) {
		log.info("START");

		DispositivoDTO dispositivo = new DispositivoDTO();
		dispositivo.setD_data_ultimo_accesso(LocalDateTime.now());
		dispositivo.setT_piattaforma(request.getPiattaforma().toUpperCase());
		dispositivo.setT_spid_code(request.getSpidCode());
		dispositivo.setT_token_firebase(request.getTokenFirebase());
		dispositivo.setT_uuid(request.getUuid());

		DispositivoDTO resultDispositivo = dao.checkDispositivo(dispositivo);
		boolean esito = false;
		if (null==resultDispositivo) {
			log.info("utente con spidcode {}, dispositivo {}, uuid {} non e' presente nella tabella T_Dispositivo",
					dispositivo.getT_spid_code(), dispositivo.getT_piattaforma(), dispositivo.getT_uuid());
			if (dao.insertDispositivo(dispositivo))
				esito = true;
			else esito = false;
		}else {
			log.info("utente con spidcode {}, dispositivo {}, uuid {} e' presente nella tabella T_Dispositivo",
					dispositivo.getT_spid_code(), dispositivo.getT_piattaforma(), dispositivo.getT_uuid());
			if (dao.updatetDispositivo(dispositivo))
				esito = true;
			else esito = false;
		}
		log.info("END");
		return esito;
	}

	@Override
	public void listOfNotification() {
		try {
			log.info("START");

			String pushTitle = env.getProperty("macs.notificatore.prenotazione.pushTitle");
			String pushMessage = env.getProperty("macs.notificatore.prenotazione.pushMessage");
			String code = env.getProperty("macs.notificatore.prenotazione.code");
			HashMap<String,String> map=null;

			if (null==pushMessage || null==pushTitle || null==code)
				throw new Exception("Errore nell'invio della notifica push pushTitle o pushMessage sono null o il codice è null");

			NotificatorePushRequestDTO request = new NotificatorePushRequestDTO();
			List<PunctualPushNotificationDTO> listOfToken =new ArrayList<PunctualPushNotificationDTO>();

			List<NotificationInfoDTO> lista = dao.listOfNotification(code);

			if (null!=lista && lista.size()>0) {

				for (NotificationInfoDTO n:lista) {

					PunctualPushNotificationDTO push = new PunctualPushNotificationDTO();
					NotificatoreTokenDTO token = new NotificatoreTokenDTO();
					token.setSistema_operativo(n.getT_piattaforma());
					token.setToken(n.getT_token_firebase());
					push.setToken(token);
					push.setPushTitle(pushTitle);
					push.setPushMessage(pushMessage);
					map=new HashMap<String,String>();
					map.put(MacsEnum.CODICE.getCodice(), code);
					map.put(MacsEnum.VALORE.getCodice(), n.getN_id_prenotazione().toString());
					push.setParametriAggiuntivi(map);
					listOfToken.add(push);

				}
				request.setNotifications(listOfToken);

				NotificatorePushResponseDTO response = client.sendPushNotification(request);
				if (null==response) {
					log.warn("RESPONSE NULL");
				}
			}else {
				log.info("NESSUNA NOTIFICA DA INVIARE");
			}
			log.info("END");

		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
		}
	}

	@Override
	public boolean logoutDispositivo(LogoutDispositivoRequestDTO request) {
		try {
			log.info("START");

			DispositivoDTO dispositivo = new DispositivoDTO();

			dispositivo.setT_uuid(request.getUuid());
			dispositivo.setT_spid_code(request.getSpidCode());
			dispositivo.setT_piattaforma(request.getPiattaforma().toUpperCase());

			log.info("END");

			return dao.logoutDispositivo(dispositivo);
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return false;
		}
	}

	@Override
	public void remainderToUse() {
		try {
			log.info("START");

			String pushTitle = env.getProperty("macs.notificatore.remebertouse.pushTitle");
			String pushMessage = env.getProperty("macs.notificatore.remebertouse.pushMessage");
			String code = env.getProperty("macs.notificatore.remebertouse.code");

			if (null==pushMessage || null==pushTitle || null==code)
				throw new Exception("Errore nell'invio della notifica push pushTitle o pushMessage sono null o il codice è null");

			List<NotificationInfoDTO> lista = dao.remainderToUse(code);

			if (null!=lista && lista.size()>0) {


				NotificatorePushBrodCastRequestDTO clientRequest = new  NotificatorePushBrodCastRequestDTO();
				List<NotificatoreTokenDTO> listOfToken= new ArrayList<NotificatoreTokenDTO>();

				for (NotificationInfoDTO n:lista) {
					NotificatoreTokenDTO token= new NotificatoreTokenDTO();
					token.setSistema_operativo(n.getT_piattaforma());
					token.setToken(n.getT_token_firebase());
					listOfToken.add(token);
				}
				clientRequest.setListOfToken(listOfToken);
				clientRequest.setPushTitle(pushTitle);
				clientRequest.setPushMessage(pushMessage);

				if (null==code || StringUtils.isBlank(code)) {
					log.error("ERRORE: codice per la push mancante dal property");
					return;
				}

				Map<String,String> parametriAggiuntivi = new HashMap<String,String>();
				parametriAggiuntivi.put(MacsEnum.CODICE.getCodice(), code);
				clientRequest.setParametriAggiuntivi(parametriAggiuntivi);

				NotificatorePushResponseDTO responseClient = client.sendPushNotification(clientRequest);
				if (null==responseClient) {
					log.warn("RESPONSE NULL");
				}
			}else {
				log.info("NESSUN UTENTE DA AVVERTIRE");
			}

			log.info("END");
			return;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return;
		}	
	}
	
	@Override
	public boolean sendPushGestorePratiche(GestorePraticaSendPushRequestDTO request) {
		try {
			log.info("START");

			String pushTitle = env.getProperty("macs.notificatore.sendPushGestorePratiche.pushTitle");
			String pushMessage = env.getProperty("macs.notificatore.sendPushGestorePratiche.pushMessage");
			String code = env.getProperty("macs.notificatore.sendPushGestorePratiche.code");

			if (null==pushMessage || null==pushTitle || null==code)
				throw new Exception("Errore nell'invio della notifica push pushTitle o pushMessage sono null o il codice è null");

			String spidCode = request.getSpidCode();
			
			List<NotificationInfoDTO> lista = dao.listOfAllAvaibleUserDevices(spidCode, code);

			if (null!=lista && lista.size()>0) {


				NotificatorePushBrodCastRequestDTO clientRequest = new  NotificatorePushBrodCastRequestDTO();
				List<NotificatoreTokenDTO> listOfToken= new ArrayList<NotificatoreTokenDTO>();

				for (NotificationInfoDTO n:lista) {
					NotificatoreTokenDTO token= new NotificatoreTokenDTO();
					token.setSistema_operativo(n.getT_piattaforma());
					token.setToken(n.getT_token_firebase());
					listOfToken.add(token);
				}
				clientRequest.setListOfToken(listOfToken);
				clientRequest.setPushTitle(pushTitle);
				clientRequest.setPushMessage(pushMessage);

				if (null==code || StringUtils.isBlank(code)) {
					log.error("ERRORE: codice per la push mancante dal property");
					return false;
				}

				Map<String,String> parametriAggiuntivi = new HashMap<String,String>();
				parametriAggiuntivi.put(MacsEnum.CODICE.getCodice(), code);
				parametriAggiuntivi.put("cf_utente", request.getCodiceFiscale());
				parametriAggiuntivi.put("valore", request.getNumPratica());
				clientRequest.setParametriAggiuntivi(parametriAggiuntivi);

				NotificatorePushResponseDTO responseClient = client.sendPushNotification(clientRequest);
				if (null==responseClient) {
					log.warn("RESPONSE NULL");
					log.info("END");
					return false;
				}
			}else {
				log.info("NESSUN UTENTE DA AVVERTIRE");
			}

			log.info("END");
			return true;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return false;
		}	
	}

	@Override
	public boolean sendPushNews(String idNews) {
		try {
			log.info("START");

			String pushTitle = env.getProperty("macs.notificatore.sendPushNews.pushTitle");
			String pushMessage = env.getProperty("macs.notificatore.sendPushNews.pushMessage");
			String code = env.getProperty("macs.notificatore.sendPushNews.code");

			if (null==pushMessage || null==pushTitle || null==code)
				throw new Exception("Errore nell'invio della notifica push pushTitle o pushMessage sono null o il codice è null");
			
			List<NotificationInfoDTO> lista = dao.listOfAllLoggedAvaibleDevice(code);

			if (null!=lista && lista.size()>0) {
				NotificatorePushBrodCastRequestDTO clientRequest = new  NotificatorePushBrodCastRequestDTO();
				List<NotificatoreTokenDTO> listOfToken= new ArrayList<NotificatoreTokenDTO>();

				for (NotificationInfoDTO n:lista) {
					NotificatoreTokenDTO token= new NotificatoreTokenDTO();
					token.setSistema_operativo(n.getT_piattaforma());
					token.setToken(n.getT_token_firebase());
					listOfToken.add(token);
				}
				clientRequest.setListOfToken(listOfToken);
				clientRequest.setPushTitle(pushTitle);
				clientRequest.setPushMessage(pushMessage);

				if (null==code || StringUtils.isBlank(code)) {
					log.error("ERRORE: codice per la push mancante dal property");
					return false;
				}

				Map<String,String> parametriAggiuntivi = new HashMap<String,String>();
				parametriAggiuntivi.put(MacsEnum.CODICE.getCodice(), code);
				parametriAggiuntivi.put(MacsEnum.VALORE.getCodice(), idNews);
				clientRequest.setParametriAggiuntivi(parametriAggiuntivi);

				NotificatorePushResponseDTO responseClient = client.sendPushNotification(clientRequest);
				if (null==responseClient) {
					log.warn("RESPOSNE NULL");
					log.info("END");
					return false;
				}
			}else {
				log.info("NESSUN UTENTE DA AVVERTIRE");
			}

			log.info("END");
			return true;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return false;
		}	
	}
	
	private boolean insertUtenzaByRuolo(String t_id_user) {
		log.info("START");
		log.info("Ruolo cittadino: {}", ruoloCittadino);
		
		var insertUtenzaRequest = new ProfilatoreInsertUtenzaByRuoloRequestDTO();
		var insertUtenzaResponse = new ProfilatoreInsertUtenzaByRuoloResponseDTO();
		
		insertUtenzaRequest.setT_id_user(t_id_user);
		insertUtenzaRequest.setT_nome_ruolo(ruoloCittadino);
		
		insertUtenzaResponse = profilatoreClient.insertUtenzaByRuolo(insertUtenzaRequest);
		
		if(Boolean.FALSE.equals(insertUtenzaResponse.isSuccess())) {
			log.info("Inserimento non avvenuto, info: {}", insertUtenzaResponse.getDescrizione());
			log.info("END");
			return false;
		}
		
		log.info("Inserimento avvenuto con successo, info: {}", insertUtenzaResponse.getDescrizione());
		log.info("END");
		return true;
	}

}
