package it.itsvil.macs.backend.util;

import java.lang.reflect.Type;
import java.util.HashMap;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import it.itsvil.macs.backend.dto.MacsTokenResponseDTO;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
public class MacsTokenUtil {
	
	@Autowired
	private Environment env;
	
	public String generateToken() {
		log.info("START");
		
		String token = new String();
		
		try {
			MacsTokenResponseDTO bearerToken = getBearerToken();
			token = bearerToken.getBearerToken().getMaggioliApiToken();
			
		} catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}
		
		log.info("END");
		return token;
	}
	
	private MacsTokenResponseDTO getBearerToken() throws Exception {
		log.info("START");
		
		MacsTokenResponseDTO tokenResponseDTO = new MacsTokenResponseDTO();
		Type responseType = new TypeToken<MacsTokenResponseDTO>(){
			private static final long serialVersionUID = 7584547924881985506L;
			}.getType();
		
		String methodName = env.getProperty(MacsEnum.MACS_TOKEN_RESOURCE.getCodice());
		String httpMethod = MacsEnum.MACS_CLIENT_METHOD_HTTP_POST.getCodice();
		String request = env.getProperty(MacsEnum.MACS_TOKEN_REQUEST_BODY.getCodice());
		
		Response response = getResponse(methodName, httpMethod, null, request);
		
		String jsonString = response.readEntity(String.class);
		
		jsonString = jsonString.replace("\"{", "{");
		jsonString = jsonString.replace("\\", StringUtils.EMPTY);
		jsonString = jsonString.replace("}\"", "}");
		log.info("jsonString: {}", jsonString);
		
		tokenResponseDTO = new Gson().fromJson(jsonString, responseType);
		log.info("tokenResponse: {}", new Gson().toJson(tokenResponseDTO));
		
		log.info("END");
		return tokenResponseDTO;
	}
	
	private Response getResponse(String servicePath, String httpMethod, HashMap<String, String> queryParam, Object request) throws Exception{
    	Response response = null;
    	
    	String baseUrl = env.getProperty(MacsEnum.MACS_TOKEN_BASE_ENDPOINT.getCodice());
		if(StringUtils.isBlank(baseUrl)) {
			log.error("ERRORE NEL RECUPERO DELL'URL");
			throw new Exception("Macs base endpoint is empty!");
		}
		
		StringBuilder url = new StringBuilder(baseUrl)
				.append(servicePath);
		
		log.info("URL: {}", url.toString());
		
		Client client = ClientBuilder.newClient();
		WebTarget webTarget = client.target(url.toString());
		
		Invocation.Builder invocationBuilder = webTarget.request().header("cache-control", "no-cache");
    	
		if (MacsEnum.MACS_CLIENT_METHOD_HTTP_GET.getCodice().equalsIgnoreCase(httpMethod)) {
			log.info("Http method: GET");
			
			if(null != queryParam && !queryParam.isEmpty()) {
				queryParam.forEach((key, value) -> webTarget.queryParam(key, value));
			}
			
			response = invocationBuilder.get();
		}
		else if (MacsEnum.MACS_CLIENT_METHOD_HTTP_POST.getCodice().equalsIgnoreCase(httpMethod)) {
			log.info("Http method: POST");
			
			response = invocationBuilder.post(Entity.entity(request, MediaType.APPLICATION_JSON_VALUE));
		}
		else if (MacsEnum.MACS_CLIENT_METHOD_HTTP_PUT.getCodice().equalsIgnoreCase(httpMethod)) {
			log.info("Http method: PUT");
			
			response = invocationBuilder.put(Entity.entity(request, MediaType.APPLICATION_JSON_VALUE));
		}
		
		log.info("RESPONSE STATUS: {}", response.getStatus());
		
		checkStatus(response);
		
		return response;
    }
	
	private void checkStatus(Response response) throws Exception {
    	if(response.getStatus() >= 200 && response.getStatus() <= 300) {
    		return;
		}else if(response.getStatus() == HttpStatus.SC_UNAUTHORIZED){
			throw new Exception("Unauthorized");
		}else if(response.getStatus() == HttpStatus.SC_EXPECTATION_FAILED){
			throw new Exception("Impossibile generare il token");
		}else if(response.getStatus() == HttpStatus.SC_BAD_REQUEST){
			throw new Exception("Bad Request");
		}else if(response.getStatus() == HttpStatus.SC_INTERNAL_SERVER_ERROR){
			throw new Exception("Internal Server Error");
		}else if(response.getStatus() == HttpStatus.SC_SERVICE_UNAVAILABLE){
			throw new Exception("Servizio momentaneamente non disponibile");
		}
    }

}
