package it.itsvil.macs.rest;

import java.util.ArrayList;
import java.util.List;

import it.itsvil.macs.backend.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import it.itsvil.macs.backend.service.ConfigurationMacsService;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value= "/configure")
public class ConfigurationMacsServiceRest {

	@Autowired
	private ConfigurationMacsService service;

	@PostMapping(value = "/1.0/insertTipoNotifiche" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertTipoNotifiche(@RequestBody TipoNotificaDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			log.info("START");
			if (null==request || null==request.getT_cod_notifiche()|| null==request.getT_nome_notifiche() || StringUtils.isBlank(request.getT_cod_notifiche()) ||  StringUtils.isBlank(request.getT_nome_notifiche())){
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				log.info("END WITH BAD REQUEST");
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}else {
				boolean risultato=service.insertTipoNotifica(request);
				if (!risultato) {
					response.setData(null);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
				response.setData("Inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
				log.info("END");				
			}
		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}

	@GetMapping(value = "/1.0/getAllTipoNotifiche" , produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<List<TipoNotificaDTO>>> getAllTipoNotifiche(){
		RisultatoDTO<List<TipoNotificaDTO>> response = new RisultatoDTO<List<TipoNotificaDTO>>();
		try {
			log.info("START");

			List<TipoNotificaDTO> risultato = service.getAllTipoNotifiche();
			if (null==risultato) {
				response.setData(null);
				response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
				log.info("END WITH EXPECTION FAIL");
				return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
			}
			response.setData(risultato);
			response.success(HttpStatus.SC_OK);
			log.info("END");				

		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}

	@GetMapping(value = "/1.0/getNotifichePerUtente" , produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<List<NotificationForUserDTO>>> getNotifichePerUtente(@RequestParam String t_spid_code){
		RisultatoDTO<List<NotificationForUserDTO>> response = new RisultatoDTO<List<NotificationForUserDTO>>();
		try {
			log.info("START");
			if (null==t_spid_code || StringUtils.isBlank(t_spid_code)){
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				log.info("END WITH BAD REQUEST");
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}else {
				 List<NotificationForUserDTO> risultato = service.getListOfAllNotificationByUser(t_spid_code);
				if (null==risultato) {
					response.setData(null);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
				response.setData(risultato);
				response.success(HttpStatus.SC_OK);
				log.info("END");				
			}
		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/updateNotifichePerUtente" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateNotifichePerUtente(@RequestBody ListNotificationForUserRequestDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			log.info("START");
			if (null==request || null==request.getSpid_code() || null==request.getListaNotifiche() || StringUtils.isBlank(request.getSpid_code())){
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				log.info("END WITH BAD REQUEST");
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}else {
				boolean risultato=service.updateNotifichePerUtente(request);
				if (!risultato) {
					response.setData(null);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
				response.setData("Aggiornamento effettuato");
				response.success(HttpStatus.SC_OK);
				log.info("END");				
			}
		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}


	@GetMapping(value = "/1.0/getPoiCategories" , produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<List<PoiConfigDTO>>> getPoiCategories(){
		RisultatoDTO<List<PoiConfigDTO>> response = new RisultatoDTO<List<PoiConfigDTO>>();
		try {
			log.info("START");
			List<PoiConfigDTO> risultato = service.getPoiCategories();
			if (null==risultato) {
				response.setData(new ArrayList<PoiConfigDTO>());
				response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
				log.info("END WITH EXPECTION FAIL");
				return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
			}
			response.setData(risultato);
			response.success(HttpStatus.SC_OK);
			log.info("END");
		}catch (Exception e) {
			e.printStackTrace();
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}

}