package it.itsvil.macs.rest;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.itsvil.macs.backend.dto.LogoutDispositivoRequestDTO;
import it.itsvil.macs.backend.dto.RisultatoDTO;
import it.itsvil.macs.backend.dto.TokenFirebaseRequestDTO;
import it.itsvil.macs.backend.service.DispositivoService;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value= "/token")
public class TokenServiceRest {
	
	@Autowired
	private DispositivoService service;
	
	@CrossOrigin
	@PostMapping(value = "/1.0/sendTokenFirebase" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	public ResponseEntity<RisultatoDTO<Boolean>> sendTokenFirebase(@RequestBody TokenFirebaseRequestDTO request){
		RisultatoDTO<Boolean> response = new RisultatoDTO<Boolean>();
		try {
			log.info("START");
			if (null==request || null==request.getPiattaforma() || null==request.getSpidCode() || null==request.getTokenFirebase() || null==request.getUuid() ||  
					StringUtils.isBlank(request.getPiattaforma()) ||  StringUtils.isBlank(request.getSpidCode())||  StringUtils.isBlank(request.getTokenFirebase()) ||
					StringUtils.isBlank(request.getUuid()) || null==request.getCodicefiscale() || StringUtils.isBlank(request.getCodicefiscale())){
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				log.info("END WITH BAD REQUEST");
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}else {
				boolean result = service.sendToken(request);
				if (!result) {
					response.setData(result);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}else {
					response.setData(result);
					response.success(HttpStatus.SC_OK);
					log.info("END");
				}
			}
		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR: {}", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@CrossOrigin
	@PostMapping(value = "/1.0/logoutDispositivo" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	public ResponseEntity<RisultatoDTO<Boolean>> logoutDispositivo(@RequestBody LogoutDispositivoRequestDTO request){
		RisultatoDTO<Boolean> response = new RisultatoDTO<Boolean>();
		try {
			log.info("START");
			if (null==request || null==request.getPiattaforma() || null==request.getSpidCode() || null==request.getUuid() ||  
					StringUtils.isBlank(request.getPiattaforma()) ||  StringUtils.isBlank(request.getSpidCode()) ||
					StringUtils.isBlank(request.getUuid())){
				response.error("Bad request", HttpStatus.SC_BAD_REQUEST);
				log.info("END WITH BAD REQUEST");
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}else {
				boolean result = service.logoutDispositivo(request);
				if (!result) {
					response.setData(result);
					response.error("Expectation Fail", HttpStatus.SC_EXPECTATION_FAILED);
					log.info("END WITH EXPECTION FAIL");
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}else {
					response.setData(result);
					response.success(HttpStatus.SC_OK);
					log.info("END");
				}
			}
		}catch (Exception e) {
			response.error("Errore", HttpStatus.SC_INTERNAL_SERVER_ERROR);
			log.error("END WITH SERVER ERROR: {}", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}

}
