package it.itsvil.notificatore.backend.dto;

import java.io.Serializable;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import io.swagger.v3.oas.annotations.media.Schema;


@JsonInclude(Include.NON_ABSENT)
public class RisultatoDTO<T> implements Serializable {

	private static final long serialVersionUID = 7721041979297547415L;
	
	@Schema(required=true, description="Esito booleano dell'operatività")
	protected boolean success;
	
	@Schema(required=true, description="Codifica dell'esito dell'operatività")
	protected int code;
	
	@Schema(required=true, description="Descrizione testuale dell'operatività")
	protected String descrizione;
	
	@Schema(required=false, description="Eventuale payload di risposta")
	protected T data;
	
	public RisultatoDTO(){
		clear();
	}
	
	public void clear(){
		this.success = false;
		this.code = 0;
		this.data = null;
		this.descrizione = null;
	}

	public boolean isSuccess() {
		return success;
	}

	public RisultatoDTO<T> success(int code) {
		this.success = true;
		this.code= code;
		return this;
	}
	
	public RisultatoDTO<T> error(String errore,int code) {
		this.success = false;
		this.descrizione = errore;
		this.code = code;
		this.data = null;
		return this;
	}
	
	public int getCode() {
		return code;
	}

	public void setCode(int code) {
		this.code = code;
	}

	public T getData() {
		return data;
	}

	public RisultatoDTO<T> setData(T data) {
		this.data = data;
		return this;
	}

	public String getDescrizione() {
		return descrizione;
	}

	public void setDescrizione(String descrizione) {
		this.descrizione = descrizione;
	}
}
