package it.itsvil.notificatore.backend.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import it.itsvil.notificatore.backend.dao.NotificatoreDao;
import it.itsvil.notificatore.backend.dto.EmailBodyDTO;
import it.itsvil.notificatore.backend.dto.EmailDTO;
import it.itsvil.notificatore.backend.dto.NotificationConfDTO;
import it.itsvil.notificatore.backend.utils.NotificatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "NotificatoreEmailServiceImpl")
public class NotificatoreEmailServiceImpl implements NotificatoreEmailService{

	@Autowired
	NotificatoreDao dao;

	@Override
	public boolean sendEmail(String n_id_app, EmailDTO email) {
		Map<String, String> emailMap = new HashMap<String, String>();
		try{
			emailMap = getEmailProperties(n_id_app);
			if(null != emailMap && !emailMap.isEmpty()){
				final String account = emailMap.get(NotificatoreEnum.EMAIL_ACCOUNT.getCodice());
				if(account == null) {
					log.info("IMPOSSIBILE RECUPERARE L'ACCOUNT");
					return false;
				}
				final String password = emailMap.get(NotificatoreEnum.EMAIL_PASSWORD.getCodice());
				if(password == null) {
					log.info("IMPOSSIBILE RECUPERARE LA PASSWORD");
					return false;
				}
				Properties props = new Properties();
				props.put(NotificatoreEnum.MAIL_SMTP_AUTH.getCodice(), "true");
				String port = emailMap.get(NotificatoreEnum.EMAIL_PORT.getCodice());
				if(port == null) {
					log.info("IMPOSSIBILE RECUPERARE LA PORTA");
					return false;
				}
				props.put(NotificatoreEnum.MAIL_SMTP_PORT.getCodice(), port);
				props.put(NotificatoreEnum.MAIL_SMTP_SOCKETFACTORY_PORT.getCodice(), port);
				props.put(NotificatoreEnum.MAIL_SMTP_SOCKETFACTORY_CLASS.getCodice(), NotificatoreEnum.JAVAX_NET_SSL_SSLSOCKETFACTORY.getCodice());

				String smtp = emailMap.get(NotificatoreEnum.EMAIL_HOST_SMTP.getCodice());
				if(smtp == null) {
					log.info("IMPOSSIBILE RECUPERARE HOST_SMTP");
					return false;
				}
				props.put(NotificatoreEnum.MAIL_SMTP_HOST.getCodice(), smtp);
				Session session = Session.getInstance(props,
						new javax.mail.Authenticator() {
							protected PasswordAuthentication getPasswordAuthentication() {
								return new PasswordAuthentication(account, password);
							}
						});

				Message message = new MimeMessage(session);

				InternetAddress from = new InternetAddress(account);

				if (email.getMittente()!=null&&!email.getMittente().trim().isEmpty()){
					message.setReplyTo(InternetAddress.parse(email.getMittente()));
				}

				if (email.getAlias()!=null&&!email.getAlias().trim().isEmpty())
					from.setPersonal(email.getAlias());


				message.setFrom(from);
				message.setSubject(email.getTextOgetto());


				EmailBodyDTO emailBody = dao.getEmailBody(n_id_app,email.getTypeBody());
				if(null == emailBody){
					return false;
				}

				String corpoEmail= new String(emailBody.getT_corpo());
				if(null != email.getPlaceHolder() && email.getPlaceHolder().size() > 0){
					int dim=email.getPlaceHolder().size();
					for (int i=0; i<dim; i++) {
						corpoEmail=corpoEmail.replace("$"+i, email.getPlaceHolder().get(i));

					}
				}

				message.setContent(corpoEmail,NotificatoreEnum.TEXT_HTML_CHARSET_UTF_8.getCodice());

				List<String> destinatari = email.getDestinatari();
				for (String d : destinatari) {
					message.setRecipients(Message.RecipientType.TO,InternetAddress.parse(d));
					Transport.send(message);
				}
			}else{
				log.info("IMPOSSIBILE RECUPERARE LE PROPRIETA' PER INVIARE LA MAIL");
				return false;
			}
			return true;
		} catch (Exception e) {
			log.error(e);
			return false;
		}

	}

	private Map<String, String> getEmailProperties(String n_id_app) {
		List<NotificationConfDTO> emailProperties = new ArrayList<NotificationConfDTO>();
		Map<String, String> map = new HashMap<String, String>();
		emailProperties = dao.getEmailProperties(n_id_app);
		if(emailProperties.size()>0) {
			for (NotificationConfDTO prop : emailProperties) {
				map.put(prop.getT_key(), prop.getT_value());
			}
			return map;
		}else {
			return null;
		}
	}
}
