package it.itsvil.profilatore.backend.service;

import java.time.LocalDateTime;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import it.itsvil.profilatore.backend.dao.ProfilatoreConfigurationDao;
import it.itsvil.profilatore.backend.dto.FunzionalitaAggiuntiveDTO;
import it.itsvil.profilatore.backend.dto.FunzionalitaDTO;
import it.itsvil.profilatore.backend.dto.FunzioneDTO;
import it.itsvil.profilatore.backend.dto.FunzioneFunzionalitaDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzionalitaAggiuntiveRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzionalitaRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzioneFunzionalitaRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzioneRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertRuoloRequestDTO;
import it.itsvil.profilatore.backend.dto.RuoloDTO;
import it.itsvil.profilatore.backend.dto.UpdateFunzionalitaRequestDTO;
import it.itsvil.profilatore.backend.dto.UpdateFunzioneRequestDTO;
import it.itsvil.profilatore.backend.dto.UpdateRuoloRequestDTO;
import it.itsvil.profilatore.backend.utils.ProfilatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "ProfilatoreConfigurationServiceImpl")
public class ProfilatoreConfigurationServiceImpl implements ProfilatoreConfigurationService{
	
	@Autowired
	ProfilatoreConfigurationDao dao;

	@Autowired
	private PlatformTransactionManager transactionManager;

	@Override
	public int insertFunzionalita(InsertFunzionalitaRequestDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			FunzionalitaDTO funzionalita = new FunzionalitaDTO();
			Long sq=dao.getSqFunzionalita();
			funzionalita.setN_id_funzionalita(sq);
			funzionalita.setT_nome(request.getT_nome_funzionalita());
			funzionalita.setT_descrizione(request.getT_descrizione_funzionalita());
			funzionalita.setD_data(LocalDateTime.now());
			funzionalita.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
			funzionalita.setT_livello_sicurezza(request.getT_livello_sicurezza());
			funzionalita.setT_info_aggiuntive(request.getT_info_aggiuntive());
			dao.insertFunzionalita(funzionalita);
			transactionManager.commit(txStatus);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}

	@Override
	public int setFunzionalita(UpdateFunzionalitaRequestDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			FunzionalitaDTO funzionalita = new FunzionalitaDTO();
			
			funzionalita.setB_attivo(request.getB_attivo());
			funzionalita.setN_id_funzionalita(request.getN_id_funzionalita());
			funzionalita.setT_descrizione(request.getT_descrizione());
			funzionalita.setT_info_aggiuntive(request.getT_info_aggiuntive());
			funzionalita.setT_livello_sicurezza(request.getT_livello_sicurezza());
			funzionalita.setT_nome(request.getT_nome());
			
			dao.setFunzionalita(funzionalita);
			transactionManager.commit(txStatus);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}




	@Override
	public Long getSequenceRuolo() {
		return dao.getSequenceRuolo();
	}

	@Override
	public int insertRuolo(InsertRuoloRequestDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			RuoloDTO ruolo = new RuoloDTO();
			ruolo.setN_id_ruolo(this.getSequenceRuolo());
			ruolo.setD_data_ins(LocalDateTime.now());
			ruolo.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
			ruolo.setN_id_app(request.getN_id_app());
			ruolo.setT_descrizione(request.getT_descrizione());
			ruolo.setT_nome(request.getT_nome());
			dao.insertRuolo(ruolo);
			transactionManager.commit(txStatus);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}

	@Override
	public int setRuolo(UpdateRuoloRequestDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			RuoloDTO ruolo = new RuoloDTO();
			ruolo.setB_attivo(request.getB_attivo());
			ruolo.setN_id_ruolo(request.getN_id_ruolo());
			ruolo.setT_descrizione(request.getT_descrizione());
			ruolo.setT_nome(request.getT_nome());
			dao.setRuolo(ruolo);
			transactionManager.commit(txStatus);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}

	@Override
	public int insertFunzione(InsertFunzioneRequestDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			FunzioneDTO funzione = new FunzioneDTO();
			LocalDateTime now =  LocalDateTime.now();
			Long n_id_funzione = dao.nextFunzioneSequence();
			
			funzione.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
			funzione.setD_data_ins(now);
			funzione.setN_id_funzione(n_id_funzione);
			funzione.setN_id_ruolo(request.getN_id_ruolo());
			funzione.setT_descrizione(request.getT_descrizione());
			funzione.setT_nome(request.getT_nome());
			dao.insertFunzione(funzione);
			transactionManager.commit(txStatus);
			return 0;
		}catch(Exception e){
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}

	@Override
	public int updateFunzione(UpdateFunzioneRequestDTO request) {
		try {
			FunzioneDTO funzione = new FunzioneDTO();
			funzione.setB_attivo(request.getB_attivo());
			funzione.setN_id_funzione(request.getN_id_funzione());
			funzione.setN_id_ruolo(request.getN_id_ruolo());
			funzione.setT_descrizione(request.getT_descrizione());
			funzione.setT_nome(request.getT_nome());
			dao.updateFunzione(funzione);
			return 0;
		}catch(Exception e){
			log.error("ERRORE: {}", e.getMessage());
			return -1;
		}
	}
	
	@Override
	public int insertFunzioneFunzionalita(InsertFunzioneFunzionalitaRequestDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			FunzioneFunzionalitaDTO ff = new FunzioneFunzionalitaDTO();
			ff.setN_id_funzione(request.getN_id_funzione());
			ff.setN_id_funzionalita(request.getN_id_funzionalita());
			ff.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
			ff.setD_data_ins(LocalDateTime.now());
			dao.insertFunzioneFunzionalita(ff);
			transactionManager.commit(txStatus);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}

	@Override
	public int setFunzioneFunzionalita(FunzioneFunzionalitaDTO request) {
		try {
			dao.setFunzioneFunzionalita(request);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return -1;
		}
	}
	
	@Override
	public int insertFunzionalitaAggiuntive(InsertFunzionalitaAggiuntiveRequestDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			FunzionalitaAggiuntiveDTO fa = new FunzionalitaAggiuntiveDTO();
			fa.setD_data_ins(LocalDateTime.now());
			fa.setB_attivo(ProfilatoreEnum.ATTIVO.getCodice());
			fa.setN_id_funzionalita(request.getN_id_funzionalita());
			fa.setN_id_funzione(request.getN_id_funzione());
			fa.setN_id_utente(request.getN_id_utente());
			dao.insertFunzionalitaAggiuntive(fa);
			transactionManager.commit(txStatus);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}
	
	@Override
	public int updateFunzionalitaAggiuntive(FunzionalitaAggiuntiveDTO request) {
		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());
		try {
			
			dao.updateFunzionalitaAggiuntive(request);
			transactionManager.commit(txStatus);
			return 0;
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus);
			return -1;
		}
	}

	@Override
	public int deleteFunzioneFunzionalita(Long n_id_funzione, Long n_id_funzionalita) {
		dao.deleteFunzioneFunzionalita(n_id_funzione,n_id_funzionalita);
		return 0;
	}

	@Override
	public int deleteFunzionalitaAggiuntive(Long n_id_funzionalita, Long n_id_utente, Long n_id_funzione) {
		dao.deleteFunzionalitaAggiuntive(n_id_funzionalita,n_id_utente,n_id_funzione);
		return 0;
	}
	
}
