package it.itsvil.profilatore.rest;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import it.itsvil.profilatore.backend.dto.DettaglioRuoloDTO;
import it.itsvil.profilatore.backend.dto.DettaglioUtenteDTO;
import it.itsvil.profilatore.backend.dto.InsertDettaglioRuoloRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertUtenzaByRuoloRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertUtenzaRequestDTO;
import it.itsvil.profilatore.backend.dto.RisultatoDTO;
import it.itsvil.profilatore.backend.dto.UpdateUtenzaRequestDTO;
import it.itsvil.profilatore.backend.service.ProfilatoreService;
import it.itsvil.profilatore.backend.utils.ProfilatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value= "/service")
public class ProfilatoreServiceRest {

	@Autowired
	ProfilatoreService service;
	
	@GetMapping(value = "/1.0/getDettaglioRuolo", produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<DettaglioRuoloDTO>> getDettaglioRuolo(@RequestHeader("ApplicationId") String applicationId, @RequestParam(name="t_nome_ruolo") String t_nome_ruolo){
		RisultatoDTO<DettaglioRuoloDTO> response = new RisultatoDTO<DettaglioRuoloDTO>();
		DettaglioRuoloDTO dettaglioRuolo = new DettaglioRuoloDTO();
		try {
			if (StringUtils.isNotBlank(t_nome_ruolo)) {
				dettaglioRuolo = service.getDettaglioRuolo(t_nome_ruolo, applicationId);
				if(null != dettaglioRuolo){
					response.setData(dettaglioRuolo);
					response.setDescrizione("Dettaglio ruolo correttamente recuperato");
					response.success(HttpStatus.SC_OK);
				}else{
					response.error("Impossibile recuperare il dettaglio del ruolo",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@GetMapping(value = "/1.0/getDettaglioUtenza", produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<DettaglioUtenteDTO>> getDettaglioUtenza(@RequestHeader("ApplicationId") String applicationId, @RequestParam(name="t_id_user") String t_id_user){
		RisultatoDTO<DettaglioUtenteDTO> response = new RisultatoDTO<DettaglioUtenteDTO>();
		DettaglioUtenteDTO dettaglioUtente = new DettaglioUtenteDTO();
		try {
			if (StringUtils.isNotBlank(t_id_user)) {
				dettaglioUtente = service.getDettaglioUtenze(t_id_user, applicationId);
				if(null != dettaglioUtente){
					response.setData(dettaglioUtente);
					response.setDescrizione("Dettaglio utente correttamente recuperato");
					response.success(HttpStatus.SC_OK);
				}else{
					response.error("Impossibile recuperare il dettaglio dell'utenza",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/updateUtenza" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateUtenza(@RequestBody UpdateUtenzaRequestDTO request, @RequestHeader("ApplicationId") String applicationId){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		
		try {
			if (null != request && StringUtils.isNotBlank(request.getT_id_user()) && StringUtils.isNotBlank(request.getB_attivo()) && 
					(request.getB_attivo().equals(ProfilatoreEnum.ATTIVO.getCodice()) || request.getB_attivo().equals(ProfilatoreEnum.NON_ATTIVO.getCodice()))) {
				int esito = service.updateUtenza(request, applicationId);
				if(esito == 0){
					response.setData("Utenza correttamente aggiornata");
					response.setDescrizione("Utenza correttamente aggiornata");
					response.success(HttpStatus.SC_OK);
				}else{
					response.error("Impossibile aggiornare l'utenza",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}

		return ResponseEntity.ok(response);

	}
	
	@PutMapping(value = "/1.0/insertDettaglioRuolo" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertDettaglioRuolo(@RequestHeader("ApplicationId") String applicationId, @RequestBody InsertDettaglioRuoloRequestDTO request){
		RisultatoDTO<String> response= new RisultatoDTO<String>();
	    
		try {
			if (null != request && StringUtils.isNotBlank(request.getT_nome_ruolo())) {
				int risp = service.insertDettaglioRuolo(request, applicationId);
				if(risp == 0) {
					response.setDescrizione("Il dettaglio del ruolo è stato correttamente inserito");
					response.success(HttpStatus.SC_OK);
					response.setData("Il ruolo "+request.getT_nome_ruolo()+" è stato correttamente inserito");
				}else if (risp == 1){
					response.error("Dettaglio ruolo già esistente",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}else{
					response.error("Impossibile inserire il dettaglio del ruolo",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PutMapping(value = "/1.0/insertUtenza", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertUtenza(@RequestBody InsertUtenzaRequestDTO request, @RequestHeader("ApplicationId")String applicationId){

		RisultatoDTO<String> response = new RisultatoDTO<String>();

		try {
			if (null != request &&  null != request.getN_id_ruolo() && StringUtils.isNotBlank(request.getT_id_user())) {
				int esito = service.insertUtenza(request, applicationId);
				if(esito == 0){
					response.setData("Utenza correttamente inserita");
					response.success(HttpStatus.SC_OK);
					response.setDescrizione("Utenza correttamente inserita");
				}else{
					response.error("Impossibile inserire l'utenza",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}

	@CrossOrigin
	@PutMapping(value = "/1.0/insertUtenzaByRuolo", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	public ResponseEntity<RisultatoDTO<String>> insertUtenzaByRuolo(
			@RequestHeader("ApplicationId") String applicationId,
			@RequestBody InsertUtenzaByRuoloRequestDTO request){
		
		log.info("***START***");

		RisultatoDTO<String> response = new RisultatoDTO<String>();

		try {
			if (null == request 
					|| StringUtils.isBlank(request.getT_id_user()) 
					|| StringUtils.isBlank(request.getT_nome_ruolo())) {
				
				log.error("BAD REQUEST");
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}	
	
			int esito = service.insertUtenzaByRuolo(request, applicationId);
				
			if(esito == -1){
				log.error("ERRORE: Inserimento non avvenuto");
				response.error("Impossibile inserire l'utenza",HttpStatus.SC_EXPECTATION_FAILED);
				return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
			}else if (esito == 1) {
				log.info("UTENZA GIA' PRESENTE");
				response.success(HttpStatus.SC_OK);
				response.setData("Utenza gia' presente");
				response.setDescrizione("Utenza gia' presente");
			}else if(esito == 0){
				log.info("INSERIMENTO AVVENUTO CON SUCCESSO");
				response.success(HttpStatus.SC_OK);
				response.setData("Utenza correttamente inserita");
				response.setDescrizione("Utenza correttamente inserita");
			}
				
			log.info("***END***");
			return ResponseEntity.ok(response);
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
	}
	
	
	
}
